==> release.12.0.73.1.tgz: cgi-cpn/lib_stats.pm <==
#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Parsing de estadísticas de búsquedas.
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
# requiere:
# use DateTime;
# use JSON;
# use glib_fildir_03;
# use glib_html_03;

use strict;
use utf8;
package lib_stats;

# elimina los logs de queries de más de 2 meses de antiguedad.
# params: void
# return: bool 0 si el dir de logs no existía o estaba vacío, 1 de otro modo.
sub rota_logs {
    my $hoy = DateTime->now();
    my $fecha_tope = $hoy->subtract(months => 2);

    my $dir_logs = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'}";
    if (!&glib_fildir_03::check_dir($dir_logs)) {
        print STDERR "En &rota_logs(), no existía directorio $dir_logs, creándolo.\n";
        return 0;
    }

    my @archivos = &glib_fildir_03::lee_dir($dir_logs);
    if (! @archivos) {
        print STDERR "En &rota_logs(), directorio de logs vacío, no se hace nada\n";
        return 0;
    }

    foreach my $archivo (@archivos) {
        next unless(-f ("$dir_logs/$archivo")); # descartamos eventuales subdirectorios.
        next unless $archivo =~ /${prontus_varglb::PRONTUS_ID}_/; # descartamos archivos que no sean logs.

        if ($archivo =~ /(\d{4})(\d{2})(\d{2}).txt$/) {
            # print STDERR "Archivo: $archivo\n";
            # my $yyyymmdd_archivo = "$1$2$3";
            # print STDERR "fecha del archivo: $yyyymmdd_archivo\n";
            my $fecha_archivo = DateTime->new(year=>$1,month=>$2,day=>$3);
            if ($fecha_archivo < $fecha_tope) {
                print STDERR "Se borra archivo $archivo, anterior a ".$fecha_tope->ymd("-")."\n";
                unlink "$dir_logs/$archivo";
            }
        }
    }
    return 1;
}

# dada una fecha en formato yyyy-mm-dd, y un período ('semana', 'mes', etc.) obtiene la fecha correspondiente al inicio de ese período.
sub get_fecha_anterior {
    my $fecha = shift;
    my $periodo = shift;

    # por defecto la semana.
    my %periodos = ('days'=>1, 'weeks'=>1, 'months'=>1);
    if (!$periodo || ! $periodos{$periodo}) {
        $periodo = 'weeks';
    }
    # generamos el objeto datetime para contar entre fechas.
    my %dt_ini;
    @dt_ini{qw[year month day]} = split(/-/, $fecha);
    my $datetime = DateTime->new(%dt_ini)->subtract($periodo => 1);
    return $datetime->ymd('-');
}

# Parsea el log del último día con búsquedas y devuelve un json procesado para presentar los datos.
sub parsea_log_mas_reciente {
    my $dir_logs = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'}";
    my ($fecha_ini, $fecha_fin);

    my $ultimo_log; # log más reciente, a presentar si no recibimos el parámetro fecha.
    my $ultimo_log_existe; # flag para marcar que ya obtuvimos el nombre del log más reciente.
    my $mostrar_logs; # flag para determinar qué logs muestro (los de una semana atrás del más reciente)
    &glib_fildir_03::check_dir($dir_logs);

    my @dirs = &glib_fildir_03::lee_dir($dir_logs);
    my @fechas_con_log;
    my ($date_fin, $date_ini);
    # Rellenamos el array @fechas_con_log y obtener la fecha de los logs más recientes.
    my $texto_log;

    foreach my $item (reverse sort @dirs) {
        next unless(-f ($dir_logs.'/'.$item));
        next unless $item =~ /${prontus_varglb::PRONTUS_ID}_/;
        last if $ultimo_log_existe;

        my ($anio, $mes, $dia);
        if ($item =~ /(\d{4})(\d{2})(\d{2}).txt$/) {
            $anio = $1;
            $mes = $2;
            $dia = $3;
            push(@fechas_con_log, "$anio-$mes-$dia");
            if (!$ultimo_log_existe) {
                $ultimo_log_existe = 1;
                $fecha_fin = $fecha_ini = "$anio-$mes-$dia"; # fecha del log más reciente
                $date_fin = $date_ini = DateTime->new("year"=>$anio,"month"=>$mes,"day"=>$dia);
            }

            my $path_log = "$dir_logs/$item"; # tomo el path del log más reciente por defecto. Este path ya está validado.

            $texto_log .= &glib_fildir_03::read_file($path_log) if ("$anio$mes$dia" >= $fecha_ini);
        }

    }

    # eliminamos los escapings.
    $texto_log =~ s/\\([rntáéíóúÁÉÍÓÚ'" +-\\])/"qq|\\$1|"/gee;

    my $queries = ();
    my $cant_queries = 0;
    my $queries_populares = ();
    # procesamos el texto como si fuera un archivo, línea por línea.
    open my $fh, '<', \$texto_log or &glib_html_03::print_json_result(0, "No se pueden procesar los datos.");
    while (my $linea = <$fh>) {
        chomp $linea;
        my @fields = split("\t", $linea);
        my $hora = substr($fields[1], 0, 2) . ':00';

        # Para el histograma de queries, evaluamos que la búsqueda tenga texto,
        # pero guardamos como dato la hora, ya que se grafica hora o fecha contra cantidad de queries.
        # Para las busquedas más populares, guardamos cada búsqueda y la cantidad de veces que se repite.
        if ($fields[3] ne '') {
            $cant_queries++;
            $queries_populares->{$fields[3]}++;
            $queries->{$hora}++;
        } else {
            print STDERR "No pude parsear la línea [$linea]\n";
        }
    }

    # ordenamos las queries más populares por cantidad y devolvemos solamente las 10 primeras.
    # ordenamos las queries más populares por cantidad y devolvemos solamente las 10 primeras.
    $queries_populares = &get_popular_queries($queries_populares, 10);

    # se duplican un poco estos datos porque algunos van directo a la página y otros van a ser procesados como json.
    my %result = (
        "queries" => $queries,
        "queries_populares" => $queries_populares,
        "total" => $cant_queries,
        "fecha_ini" => $fecha_ini,
        "fecha_fin" => $fecha_fin,
    );

    my %datos = (
        "queries" => \%result,
        "total" => $cant_queries,
        "fecha_ini" => $fecha_ini,
        "fecha_fin" => $fecha_fin,
    );

    return \%datos;
}

# recibe una fecha de log, y busca hacia atrás según el segundo parámetro (día, semana, mes, etc.)
sub parsea_logs_fechas {
    my $fecha_ini = shift;
    my $fecha_fin = shift;

    # generamos los objetos datetime de inicio y fin para contar entre fechas.
    my %dt_ini;
    @dt_ini{qw[year month day]} = split(/-/, $fecha_ini);
    my %dt_fin;
    @dt_fin{qw[year month day]} = split(/-/, $fecha_fin);
    # print STDERR Dumper(\%dt_ini);
    # print STDERR Dumper(\%dt_fin);
    my $inicio = DateTime->new(%dt_ini);
    my $fin = DateTime->new(%dt_fin);
    my $actual = $inicio;
    my $texto_log;
    my $fecha_aaaammdd;
    while ($actual <= $fin) {
        $fecha_aaaammdd = $actual->ymd(''); # el separador en este caso debe ser '', vacío.
        # print STDERR "actual: $fecha_aaaammdd\n";
        $actual->add(days=>1); # aumentamos el contador de fecha.
        my $path_log = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'}/$prontus_varglb::PRONTUS_ID\_$fecha_aaaammdd.txt";
        next if (! -f $path_log);

        $texto_log .= &glib_fildir_03::read_file($path_log);
    }

    # eliminamos los escapings.
    $texto_log =~ s/\\([rntáéíóúÁÉÍÓÚ'" +-\\])/"qq|\\$1|"/gee;

    my $queries = ();
    my $queries_populares = ();
    my $cant_queries = 0;
    # procesamos el texto como si fuera un archivo, línea por línea.
    open my $fh, '<', \$texto_log or &glib_html_03::print_json_result(0, "No se pueden procesar los datos.");
    while (my $linea = <$fh>) {
        chomp $linea;
        my @fields = split("\t", $linea);
        if ($fields[3] ne '') {
            $cant_queries++;
            $fields[0] =~ s#(\d{2})/(\d{2})/(\d{4})#$3-$2-$1#g;
            $queries->{$fields[0]}++;
            $queries_populares->{$fields[3]}++;
        } else {
            print STDERR "No pude parsear la línea [$linea]\n";
        }
    }
    # ordenamos las queries más populares por cantidad y devolvemos solamente las 10 primeras.
    $queries_populares = &get_popular_queries($queries_populares, 10);

    # se duplican un poco estos datos porque algunos van directo a la página y otros van a ser procesados como json.
    my %result = (
        "queries" => $queries,
        "queries_populares" => $queries_populares,
        "total" => $cant_queries,
        "fecha_ini" => $fecha_ini,
        "fecha_fin" => $fecha_fin,
    );

    return \%result;
}

# devuelve un hash con las N queries más buscadas
sub get_popular_queries {
    my ($queries, $cant) = @_;
    return {} if (!defined $queries);
    my %queries = %$queries;

    # ordenamos las queries más populares por cantidad y devolvemos solamente las N primeras.
    my @queries_ordenadas = reverse sort { $queries{$a} <=> $queries{$b} } keys %queries;
    my $len = @queries_ordenadas;
    if ($len > $cant) { $len = $cant; }
    @queries_ordenadas = @queries_ordenadas[0..$len-1];
    my %tmp = map { $_ => $queries{$_} if ($_ ne '')} @queries_ordenadas;
    %queries = %tmp;
    return \%queries;
}

1;

==> release.12.2.28.1.tgz: cgi-cpn/lib_stats.pm <==
#!/usr/bin/perl

# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
# -------------------------------COMENTARIO GLOBAL---------------
# ---------------------------------------------------------------
# PROPOSITO.
# -----------
# Parsing de estadísticas de búsquedas.
# ---------------------------------------------------------------
# ---------------------------------------------------------------
# DIRECTIVAS DE COMPILACION.
# ---------------------------
# requiere:
# use DateTime;
# use glib_fildir_03;
# use glib_html_03;

use strict;
use utf8;
package lib_stats;

# elimina los logs de queries de más de 2 meses de antiguedad.
# params: void
# return: bool 0 si el dir de logs no existía o estaba vacío, 1 de otro modo.
sub rota_logs {
    my $hoy = DateTime->now();
    my $fecha_tope = $hoy->subtract(months => 2);

    my $dir_logs = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'}";
    if (!glib_fildir_03::check_dir($dir_logs)) {
        print STDERR "En &rota_logs(), no existía directorio $dir_logs, creándolo.\n";
        return 0;
    }

    my @archivos = glib_fildir_03::lee_dir($dir_logs);
    if (! @archivos) {
        print STDERR "En &rota_logs(), directorio de logs vacío, no se hace nada\n";
        return 0;
    }

    foreach my $archivo (@archivos) {
        next unless(-f ("$dir_logs/$archivo")); # descartamos eventuales subdirectorios.
        next unless $archivo =~ /${prontus_varglb::PRONTUS_ID}_/; # descartamos archivos que no sean logs.

        if ($archivo =~ /(\d{4})(\d{2})(\d{2}).txt$/) {
            # print STDERR "Archivo: $archivo\n";
            # my $yyyymmdd_archivo = "$1$2$3";
            # print STDERR "fecha del archivo: $yyyymmdd_archivo\n";
            my $fecha_archivo = DateTime->new(year=>$1,month=>$2,day=>$3);
            if ($fecha_archivo < $fecha_tope) {
                print STDERR "Se borra archivo $archivo, anterior a ".$fecha_tope->ymd("-")."\n";
                unlink "$dir_logs/$archivo";
            }
        }
    }
    return 1;
}

# dada una fecha en formato yyyy-mm-dd, y un período ('semana', 'mes', etc.) obtiene la fecha correspondiente al inicio de ese período.
sub get_fecha_anterior {
    my $fecha = shift;
    my $periodo = shift;

    # por defecto la semana.
    my %periodos = ('days'=>1, 'weeks'=>1, 'months'=>1);
    if (!$periodo || ! $periodos{$periodo}) {
        $periodo = 'weeks';
    }
    # generamos el objeto datetime para contar entre fechas.
    my %dt_ini;
    @dt_ini{qw[year month day]} = split(/-/, $fecha);
    my $datetime = DateTime->new(%dt_ini)->subtract($periodo => 1);
    return $datetime->ymd('-');
}

# Parsea el log del último día con búsquedas y devuelve un json procesado para presentar los datos.
sub parsea_log_mas_reciente {
    my $dir_logs = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'}";
    my ($fecha_ini, $fecha_fin);

    my $ultimo_log; # log más reciente, a presentar si no recibimos el parámetro fecha.
    my $ultimo_log_existe; # flag para marcar que ya obtuvimos el nombre del log más reciente.
    my $mostrar_logs; # flag para determinar qué logs muestro (los de una semana atrás del más reciente)
    glib_fildir_03::check_dir($dir_logs);

    my @dirs = glib_fildir_03::lee_dir($dir_logs);
    my @fechas_con_log;
    my ($date_fin, $date_ini);
    # Rellenamos el array @fechas_con_log y obtener la fecha de los logs más recientes.
    my $texto_log;

    foreach my $item (reverse sort @dirs) {
        next unless(-f ($dir_logs.'/'.$item));
        next unless $item =~ /${prontus_varglb::PRONTUS_ID}_/;
        last if $ultimo_log_existe;

        my ($anio, $mes, $dia);
        if ($item =~ /(\d{4})(\d{2})(\d{2}).txt$/) {
            $anio = $1;
            $mes = $2;
            $dia = $3;
            push(@fechas_con_log, "$anio-$mes-$dia");
            if (!$ultimo_log_existe) {
                $ultimo_log_existe = 1;
                $fecha_fin = $fecha_ini = "$anio-$mes-$dia"; # fecha del log más reciente
                $date_fin = $date_ini = DateTime->new("year"=>$anio,"month"=>$mes,"day"=>$dia);
            }

            my $path_log = "$dir_logs/$item"; # tomo el path del log más reciente por defecto. Este path ya está validado.

            $texto_log .= glib_fildir_03::read_file($path_log) if ("$anio$mes$dia" >= $fecha_ini);
        }

    }

    # eliminamos los escapings.
    $texto_log =~ s/\\([rntáéíóúÁÉÍÓÚ'" +-\\])/"qq|\\$1|"/gee;

    my $queries = ();
    my $cant_queries = 0;
    my $queries_populares = ();
    # procesamos el texto como si fuera un archivo, línea por línea.
    open my $fh, '<', \$texto_log or glib_html_03::print_json_result(0, "No se pueden procesar los datos.");
    while (my $linea = <$fh>) {
        chomp $linea;
        my @fields = split("\t", $linea);
        my $hora = substr($fields[1], 0, 2) . ':00';

        # Para el histograma de queries, evaluamos que la búsqueda tenga texto,
        # pero guardamos como dato la hora, ya que se grafica hora o fecha contra cantidad de queries.
        # Para las busquedas más populares, guardamos cada búsqueda y la cantidad de veces que se repite.
        if ($fields[3] ne '') {
            $cant_queries++;
            $queries_populares->{$fields[3]}++;
            $queries->{$hora}++;
        } else {
            print STDERR "No pude parsear la línea [$linea]\n";
        }
    }

    # ordenamos las queries más populares por cantidad y devolvemos solamente las 10 primeras.
    # ordenamos las queries más populares por cantidad y devolvemos solamente las 10 primeras.
    $queries_populares = &get_popular_queries($queries_populares, 10);

    # se duplican un poco estos datos porque algunos van directo a la página y otros van a ser procesados como json.
    my %result = (
        "queries" => $queries,
        "queries_populares" => $queries_populares,
        "total" => $cant_queries,
        "fecha_ini" => $fecha_ini,
        "fecha_fin" => $fecha_fin,
    );

    my %datos = (
        "queries" => \%result,
        "total" => $cant_queries,
        "fecha_ini" => $fecha_ini,
        "fecha_fin" => $fecha_fin,
    );

    return \%datos;
}

# recibe una fecha de log, y busca hacia atrás según el segundo parámetro (día, semana, mes, etc.)
sub parsea_logs_fechas {
    my $fecha_ini = shift;
    my $fecha_fin = shift;

    # generamos los objetos datetime de inicio y fin para contar entre fechas.
    my %dt_ini;
    @dt_ini{qw[year month day]} = split(/-/, $fecha_ini);
    my %dt_fin;
    @dt_fin{qw[year month day]} = split(/-/, $fecha_fin);
    # print STDERR Dumper(\%dt_ini);
    # print STDERR Dumper(\%dt_fin);
    my $inicio = DateTime->new(%dt_ini);
    my $fin = DateTime->new(%dt_fin);
    my $actual = $inicio;
    my $texto_log;
    my $fecha_aaaammdd;
    while ($actual <= $fin) {
        $fecha_aaaammdd = $actual->ymd(''); # el separador en este caso debe ser '', vacío.
        # print STDERR "actual: $fecha_aaaammdd\n";
        $actual->add(days=>1); # aumentamos el contador de fecha.
        my $path_log = "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_LOG_ES'}/$prontus_varglb::PRONTUS_ID\_$fecha_aaaammdd.txt";
        next if (! -f $path_log);

        $texto_log .= glib_fildir_03::read_file($path_log);
    }

    # eliminamos los escapings.
    $texto_log =~ s/\\([rntáéíóúÁÉÍÓÚ'" +-\\])/"qq|\\$1|"/gee;

    my $queries = ();
    my $queries_populares = ();
    my $cant_queries = 0;
    # procesamos el texto como si fuera un archivo, línea por línea.
    open my $fh, '<', \$texto_log or glib_html_03::print_json_result(0, "No se pueden procesar los datos.");
    while (my $linea = <$fh>) {
        chomp $linea;
        my @fields = split("\t", $linea);
        if ($fields[3] ne '') {
            $cant_queries++;
            $fields[0] =~ s#(\d{2})/(\d{2})/(\d{4})#$3-$2-$1#g;
            $queries->{$fields[0]}++;
            $queries_populares->{$fields[3]}++;
        } else {
            print STDERR "No pude parsear la línea [$linea]\n";
        }
    }
    # ordenamos las queries más populares por cantidad y devolvemos solamente las 10 primeras.
    $queries_populares = &get_popular_queries($queries_populares, 10);

    # se duplican un poco estos datos porque algunos van directo a la página y otros van a ser procesados como json.
    my %result = (
        "queries" => $queries,
        "queries_populares" => $queries_populares,
        "total" => $cant_queries,
        "fecha_ini" => $fecha_ini,
        "fecha_fin" => $fecha_fin,
    );

    return \%result;
}

# devuelve un hash con las N queries más buscadas
sub get_popular_queries {
    my ($queries, $cant) = @_;
    return {} if (!defined $queries);
    my %queries = %$queries;

    # ordenamos las queries más populares por cantidad y devolvemos solamente las N primeras.
    my @queries_ordenadas = reverse sort { $queries{$a} <=> $queries{$b} } keys %queries;
    my $len = @queries_ordenadas;
    if ($len > $cant) { $len = $cant; }
    @queries_ordenadas = @queries_ordenadas[0..$len-1];
    my %tmp = map { $_ => $queries{$_} if ($_ ne '')} @queries_ordenadas;
    %queries = %tmp;
    return \%queries;
}

return 1;
